#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define VPCALC_SECTION	"MEM"
#define VPCALC_SUBSECT	"RAW16"

#include "../config.h"
#include "../include/types.h"
#include "../include/debug.h"

#include "../include/memory.h"

typedef struct _memory_o {
	union {
		uint8 mem8[0x10000];
  		uint16 mem16[0x8000];
	} data;
} _memory_o;

memory_o memory_new(void)
{
	_memory_o *new;

	new = (_memory_o *) malloc(sizeof(_memory_o));

	if (new != NULL) {
		memset(new, 0, sizeof(_memory_o));
	}

	return (memory_o)new;
}

void memory_del(memory_o memory)
{
	free(memory);
}

size_t memory_size(memory_o memory)
{
	_memory_o *_memory = (_memory_o *)memory;

	return sizeof(_memory->data.mem8); 
}

size_t memory_bus_size(memory_o memory)
{
	return 16;
}

size_t memory_addr_size(memory_o memory)
{
	return 16;
}

void memory_putimage(memory_o memory, void *data)
{
	_memory_o *_memory = (_memory_o *)memory;

	memcpy(_memory->data.mem8, data, sizeof(_memory->data.mem8));
}

void memory_getimage(memory_o memory, void *data)
{
	_memory_o *_memory = (_memory_o *)memory;

	memcpy(data, _memory->data.mem8, sizeof(_memory->data.mem8));
}

uint memory_fetch(memory_o memory, uint addr)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr = ((addr & 0xffff) >> 1);


#if HOST_ENDIANESS == HOST_ENDIANESS_BE
	return _memory->data.mem16[addr];
#else
	uint16 ret;

	ret = _memory->data.mem16[addr];

	return ((ret & 0x00ff) << 8) | ((ret & 0xff00) >> 8);
#endif
}

uint memory_read(memory_o memory, uint addr)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr = ((addr & 0xffff) >> 1);


#if HOST_ENDIANESS == HOST_ENDIANESS_BE
	return _memory->data.mem16[addr];
#else
	uint16 ret;

	ret = _memory->data.mem16[addr];

	return ((ret & 0x00ff) << 8) | ((ret & 0xff00) >> 8);
#endif
}

void memory_write(memory_o memory, uint addr, uint data)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr = ((addr & 0xffff) >> 1);

#if HOST_ENDIANESS == HOST_ENDIANESS_LE
	data = ((data & 0x00ff) << 8) | ((data & 0xff00) >> 8);
#endif

	_memory->data.mem16[addr] = data;
}

uint8 memory_fetch_byte(memory_o memory, uint addr)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr &= 0xffff;

	return _memory->data.mem8[addr];
}

uint8 memory_read_byte(memory_o memory, uint addr)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr &= 0xffff;

	return _memory->data.mem8[addr];
}

void memory_write_byte(memory_o memory, uint addr, uint8 data)
{
	_memory_o *_memory = (_memory_o *)memory;

	addr &= 0xffff;

	_memory->data.mem8[addr] = data;
}

void memory_remap(memory_o memory, uint page, uint addr, uint perm)
{
	/* This function intentionally left blank. */
	return;
}

int memory_raw(memory_o memory, void **rd_ptr, void **wr_ptr, void **ex_ptr)
{
	_memory_o *_memory = (_memory_o *)memory;

	*rd_ptr = (void *)_memory->data.mem8;
	*wr_ptr = (void *)_memory->data.mem8;
	*ex_ptr = (void *)_memory->data.mem8;

	return 1;
}
