#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define VPCALC_SECTION	"VPCALC"
#define VPCALC_SUBSECT	"CONF"

#include "../config.h"
#include "../include/types.h"
#include "../include/debug.h"

#include "../include/conf.h"

#define MAX_KEYS	1024

typedef struct _config_o {
	const char *filename;

	unsigned int num_keys;
	const char *keys[MAX_KEYS];
	const char *vals[MAX_KEYS];
} _config_o;

config_o config_open(const char *filename)
{
	_config_o *new;
	char buffer[2048];
	char *key, *val;
	char *p;
	FILE *input;
	unsigned int cur = 0;

	input = fopen(filename, "r");
	if (input == NULL) {
		_err_report("Unable to open config file.");
		return NULL;
	}

	new = (_config_o *)malloc(sizeof(_config_o));
	if (new == NULL) {
		_err_report("Unable to allocate config structure.");
		fclose(input);
		return NULL;
	}

	memset(new, 0, sizeof(_config_o));

	new->filename = strdup(filename);

	while (!feof(input)) {
		if (cur > MAX_KEYS) {
			_warn_report("Too many config values!");
			break;
		}

		if (fgets(buffer, 1024, input) == NULL) {
			break;
		}

		key = buffer;

		p = key;
		while (*p && *p != ';' && *p != '\n')
			p++;
		*p = 0;

		while (*key == ' ' || *key == '\t')
			key++;

		if (*key == 0)
			continue;

		val = key;

		while (*val && *val != ':')
			val++;

		p = val - 1;
		while (*p == ' ' || *p == '\t')
			*(p--) = 0;

		if (*val != ':') {
			val = "true";
		} else {
			*(val++) = 0;
		}

		while (*val == ' ' || *val == '\t')
			val++;

		p = val;
		while (*p) p++;

		p--;
		while (p != val && (*p == ' ' || *p == '\t'))
			*(p--) = 0;

		new->keys[cur] = strdup(key);
		new->vals[cur] = strdup(val);
		cur++;
	}

	new->num_keys = cur;

	fclose(input);

	return new;
}

void config_close(config_o conf)
{
	_config_o *_conf = (_config_o *)conf;

	free(_conf);
}

int config_isset(config_o conf, const char *key)
{
	_config_o *_conf = (_config_o *)conf;
	int i;

	for (i=0; i<_conf->num_keys; i++) {
		if(!strcmp(_conf->keys[i], key)) {
			return 1;
		}
	}

	return 0;
}
const char *config_string(config_o conf, const char *key)
{
	_config_o *_conf = (_config_o *)conf;
	int i;

	for (i=0; i<_conf->num_keys; i++) {
		if(!strcmp(_conf->keys[i], key)) {
			return _conf->vals[i];
		}
	}

	return "";
}

int config_integer(config_o conf, const char *key)
{
	_config_o *_conf = (_config_o *)conf;
	int i;

	for (i=0; i<_conf->num_keys; i++) {
		if (!strcmp(_conf->keys[i], key)) {
			if (!strcmp(_conf->vals[i], "true")) {
				return 1;
			}

			return strtol(_conf->vals[i], NULL, 0);
		}
	}

	return 0;
}
